<?php

namespace Tests\Feature;

use App\Models\Server;
use App\Models\User;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Tests\TestCase;
use Illuminate\Support\Facades\Hash;

class ServerLoginTest extends TestCase
{
    use RefreshDatabase;

    public function test_empty() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $response = $this->postJson("/api/auth/login", []);
        $response->assertStatus(422);
    }

    public function test_wrong_password() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $response = $this->postJson("/api/auth/server/login", [
            "login" => "test",
            "password" => "123"
        ]);
        $response->assertStatus(401);
    }

    public function test_good() {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $response = $this->postJson("/api/auth/server/login", [
            "login" => "test",
            "password" => "test"
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "token",
            "server" => [
                "id",
                "login",
                "public",
                "created_at",
                "updated_at",
                "owner_id"
            ]
        ]);
    }

    public function test_store_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->post('/api/server', [
            "login" => "test",
            "password" => "testtest",
            "public" => 1
        ]);
        $response->assertStatus(201);
        $this->assertDatabaseCount('servers', 1);

        $response = $this->postJson("/api/auth/server/login", [
            "login" => "test",
            "password" => "testtest"
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "token",
            "server" => [
                "id",
                "login",
                "public",
                "created_at",
                "updated_at",
                "owner_id"
            ]
        ]);
    }

    public function test_update_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);

        $server = Server::create([
            'login' => 'test',
            'password' => Hash::make('test'),
            'owner_id' => $user->id,
            'public' => true
        ]);

        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->put('/api/server/' . $server->id, [
            "password" => "testtest",
        ]);
        $response->assertStatus(204);
        $this->assertDatabaseCount('servers', 1);

        $response = $this->postJson("/api/auth/server/login", [
            "login" => "test",
            "password" => "testtest"
        ]);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "token",
            "server" => [
                "id",
                "login",
                "public",
                "created_at",
                "updated_at",
                "owner_id"
            ]
        ]);
    }
}
