<?php

namespace Tests\Feature;

use App\Models\Server;
use App\Models\User;
use App\Models\Game;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Tests\TestCase;

class UserTest extends TestCase
{
    use RefreshDatabase;

    public function test_index()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/user');
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "users" => [
                '*' => [
                    "id",
                    "username",
                    "created_at",
                    "updated_at"
                ]
            ]
        ]);
    }

    public function test_show_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/user/999');
        $response->assertStatus(404);
    }

    public function test_show_good()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/user/' . $user->id);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "user" => [
                "id",
                "username",
                "created_at",
                "updated_at"
            ]
        ]);

    
    }
    public function test_index_usergame_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/user/999/game');
        $response->assertStatus(404);
    }

    public function test_index_usergame_good()
    {

        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "public" => 1,
            "owner_id" => $user->id
        ]);

        $game = Game::create([
            'server_id' => $server->id,
        ]);

        $game->users()->attach($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/user/' . $user->id . '/game');
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "games" => [
                '*' => [
                    "id",
                    "server_id",
                    "created_at",
                    "updated_at"
                ]
            ]
        ]);
    }

    public function test_show_usergame_not_found()
    {
        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/user/999/game/999');
        $response->assertStatus(404);
    }

    public function test_show_usergame_good()
    {

        $user = User::create([
            'username' => 'test',
            'email' => 'test@localhost',
            'password' => Hash::make('test'),
        ]);
        $token = Auth::login($user);

        $server = Server::create([
            "login" => "test",
            "password" => "test",
            "public" => 1,
            "owner_id" => $user->id
        ]);

        $game = Game::create([
            'server_id' => $server->id,
        ]);

        $game->users()->attach($user);

        $response = $this->withHeaders([
            "Authorization" => "Bearer " . $token
        ])->get('/api/user/' . $user->id . '/game/' . $game->id);
        $response->assertStatus(200);
        $response->assertJsonStructure([
            "game" => [
                "id",
                "server_id",
                "created_at",
                "updated_at"
            ]
        ]);
    }
}
